<?php
namespace MyMenu\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\Session\Session;
use MyMenu\Libraries\KeyValueStorage;
use MyMenu\Services\JWT;

class Informativos
{

    private $environment;
    private $dataSourceManager;
    private $parameterBagProvider;
    private $utilities;
    private $sessionManager;

    public function __construct($crudService, $environment, $dataSourceManager, $parameterBagProvider, $utilities, Session $sessionManager)
    {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->parameterBagProvider = $parameterBagProvider;
        $this->utilities = $utilities;
        $this->sessionManager = $sessionManager;
    }

    public function getInformativos(Request\Filter $request, Response $response)
    {
        $dataSourceName = 'informativomymenu';
        $requestHeaders = getallheaders();
        $token = isset($requestHeaders["token"]) ? $requestHeaders["token"] : null;
        $tokenData = JWT::getTokenData($token);
        
        $conditions = array(
            array(
                'columnName' => 'NRQRCODELOCALPESQ',
                'operator' => '=',
                'value' => $tokenData['NUMQRCODE'],
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $tokenData['NRORG'],
            ),
        );
        try {
            
            $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
            
            $rowsToReturn = $dataSet->getRows();
            for ($i = 0; $i < sizeof($rowsToReturn); $i++) {
                $this->utilities->concatFileServerPathToRow($rowsToReturn[$i], 'URLIMAGEM');
                $this->utilities->createStripedFieldInRow($rowsToReturn[$i], 'TXCONTEUDO', 'TXCONTEUDO_UNF');
            }

            $dataSetToReturn = new DataSet($dataSourceName, $rowsToReturn);
            $response->addDataSet($dataSetToReturn);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    public function lastAccessOnApp(Request $request, Response $response)
    {
        $lastAccs = $request->getRow()['lastAccs'];
        $requestHeaders = getallheaders();
        $token = isset($requestHeaders["token"]) ? $requestHeaders["token"] : null;
        $tokenData = JWT::getTokenData($token);

        $dataSourceName = 'newinformativomymenu';
        $conditions = array(
            array(
                'columnName' => 'NRQRCODELOCALPESQ',
                'operator' => '=',
                'value' => $tokenData['NUMQRCODE'],
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrorg(),
            ),
            array(
                'columnName' => 'LASTACCS',
                'operator' => '=',
                'value' => $lastAccs,
            ),
        );
        try {
            $filterCriteria = new FilterCriteria($dataSourceName, $conditions);
            $newFilterCriteria = $this->parameterBagProvider->getPopulatedFilterCriteria($filterCriteria);
            $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);

            $rowsToReturn = $dataSet->getRows();
            $cont = array('count' => sizeof($rowsToReturn));
            $dataSetToReturn = new DataSet('count', $cont);
            $response->addDataSet($dataSetToReturn);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}
