var ZeedhiDirectives = angular.module("ZeedhiDirectives");

function loadImage(imageUrl, callback) {
	if (imageUrl) {
		$.ajax({
			url: imageUrl,
			cache: true,
			processData: false
		}).done(callback);
	}
}

ZeedhiDirectives.directive("zhProductGrid", function() {
	return {
		link: function($scope, element) {
			var cart = element.find('div.cart');
			var header = element.find('div.product-grid-header');

			element.on("scroll.product-grid", function() {
				cart.css('margin-top', element.scrollTop());
				header.css('margin-top', element.scrollTop());
			});

			header.find('.header-shortcut').on("change", function(e) {
				var selectedItem = $(e.currentTarget).find(':selected').val();
				if (selectedItem) {
					var group = element.find('ul.product-groups > li[data-group-code="'+selectedItem+'"]');
					var groupTop = group.position().top;
					element.animate({ scrollTop: groupTop - 60 }, 300);
				}
			});

			var cartItems = cart.find('ul.cart-items');
			cartItems.on("mousewheel DOMMouseScroll", function(e, delta) {
				if (cartItems.height() < cartItems[0].scrollHeight) {
					if ((e.originalEvent.deltaY < 0 && cartItems.scrollTop() === 0) ||
							(e.originalEvent.deltaY > 0 && cartItems.scrollTop() + cartItems.height() == cartItems[0].scrollHeight)) {
						return false;
					}
				}
			});

			$scope.toggleGroup = function(e) {
				var group = $(e.currentTarget.parentElement);
				if (group.hasClass('opened')) {
					group.css('height', (group.hasClass('no-image') ? 32 : 112) + group.find('ul.product-items').innerHeight());
					group.removeClass('opened');
					group.animate({'height': 32}, 300);
				} else {
					group.css('height', 32);
					group.addClass('opened');
					group.animate({'height': (group.hasClass('no-image') ? 32 : 112) + group.find('ul.product-items').innerHeight()}, 300, function() {
						group.css('height', 'auto');
					});
				}
			};

			$scope.toggleCartItems = function(e) {
				var cart = $(e.currentTarget).parents('.cart').eq(0);
				cart.toggleClass('opened');
			};

			$scope.loadGroupImage = function(index, imageUrl) {
				var group = element.find('ul.product-groups > li').eq(index);
				loadImage(imageUrl, function() {
					var element = group.find('.group-img span');
					element.css("background-image",'url('+imageUrl+')').fadeIn();
					group.removeClass('no-image');
				});
			};

			$scope.$watch(function() { return $scope.widget.searchCriteria; }, function(val) {
				// force groups to be opened during searching
				if (val) { element.find('ul.product-groups > li').addClass('opened').css('height', 'auto'); }
			});
		}
	};
});

ZeedhiDirectives.directive("zhProductGridDetailImage", function() {
	return {
		link: function($scope, element, attrs) {
			var imageUrl = $scope.$eval(attrs.zhProductGridDetailImage);
			loadImage(imageUrl, function() {
				var span = element.find('span');
				span.css("background-image",'url('+imageUrl+')').fadeIn();
				element.parent().removeClass('no-image');
			});
		}
	};
});

ZeedhiDirectives.directive("zhProductGridDetailTab", function(templateManager, $timeout) {
	return {
		link: function($scope, element) {
			var scrollContainer = element.find('.zh-tab-scroller');

			element.on("mousewheel wheel DOMMouseScroll", function(e) {
				scrollContainer[0].scrollLeft += e.originalEvent.deltaY;
			});

			var showScrollButtons = function() {
				if (scrollContainer[0].scrollLeft > 0) {
					element.find("span.icon-scroll-left").show();
				} else {
					element.find("span.icon-scroll-left").hide();					
				}

				if (scrollContainer[0].scrollWidth - scrollContainer[0].scrollLeft > scrollContainer[0].clientWidth) {
					element.find("span.icon-scroll-right").show();					
				} else {
					element.find("span.icon-scroll-right").hide();										
				}
			};
			$timeout(showScrollButtons, 100);
			element.find(".zh-tab-scroller").on("scroll", showScrollButtons);

			$scope.scrollLeft = function() {
				scrollContainer.animate({ scrollLeft: scrollContainer[0].scrollLeft - 50}, 100);
			};

			$scope.scrollRight = function() {
				scrollContainer.animate({ scrollLeft: scrollContainer[0].scrollLeft + 50}, 100);
			};

			var object = {
				firstRefresh: 3000,
				regularRefresh: 1000,
				interval: 100,
				clearScheduler: function() {
					clearTimeout(object.timer);
					object.currentTime = 0;
					// when it's the second time running decreases the refresh rate
					// from object.firstRefresh to object.regularRefresh
					object.refresh = object.firstRun ? object.regularRefresh : object.firstRefresh;
				},
				startUpdate: function() {
					// if the productItem.selectedPage or productItem.detailPages cease to exist, do not set schedule.
					// this happens when a tabbedWidget is inside a tabbed container
					// and the container changes tabs.
					object.clearScheduler();
					if ($scope.widget.productItem.selectedPage && $scope.widget.productItem.detailPages) {
						object.setScheduler();
					}
				},
				setScheduler: function() {
					// checks if it should refresh again, and if so, increments the timer.
					// otherwise, time's reset and no other event is scheduled.
					// redundant verification for the existance of the productItem.selectedPage and productItem.detailPages objects.
					if ((object.currentTime < object.refresh) && ($scope.widget.productItem.selectedPage && $scope.widget.productItem.detailPages)) {
						object.udpateElements();
						object.timer = setTimeout(object.setScheduler, object.interval);
						object.currentTime += object.interval;
					} else {
						object.firstRun = true;
					}
				},
				udpateElements: function() {
					// does simple math to decide the size of the line and its position.
					object.elementDiv = object.elementDiv || $($(element).find('div')[0]);
					object.elementLine = object.elementLine || object.elementDiv.find('.zh-tab-line');
					object.elementCell = object.elementDiv.find('.zh-tab-cell');

					var index = $scope.widget.productItem.detailPages.indexOf($scope.widget.productItem.selectedPage);
					var elementWidth = $(object.elementCell[index]).width(),
						priorWidth = object.getPriorWidth(index);

					object.elementLine.width(elementWidth);
					object.elementLine.css('left', priorWidth);
				},
				getPriorWidth: function(index) {
					// gets the index of a tab and returns the width of its combined predecessors
					var priorWidth = 0;
					for (i = 0; i < index; i++) {
						priorWidth += $(object.elementCell[i]).width();
					}
					return index ? priorWidth + (index * 15) : priorWidth;
				}
			};

			['widget.productItem.selectedPage', 'widget.productItem.detailPages'].forEach(function(each) {
				$scope.$watch(each, object.startUpdate);
			});
		}
	};
});

